//Original Code by Kreed (derek-liauw@usa.net)
//Modified by Jean-Philip Desjardins to fit in MCHE

#include <sys/farptr.h>
#include <allegro.h>
#include "MCHE.H"

#ifdef MCHE_ASM
#define MMX
#endif

#define SHARPEN //doesn't quite improve the graphics as I thought it would, 2fps slowdown

#define uint8 unsigned char
#define uint16 unsigned short
#define uint32 unsigned long
#define int32 long

//extern uint32 screen_width;
//extern uint32 screen_height;
#define screen_width mche_ScreenX
#define screen_height mche_ScreenY

/* Macros for use with Interpolate V2 */
inline int GetResult1(uint32 A, uint32 B, uint32 C, uint32 D, uint32 E)
{
 int x = 0; 
 int y = 0;
 int r = 0;
 if (A == C) x+=1; else if (B == C) y+=1;
 if (A == D) x+=1; else if (B == D) y+=1;
 if (x <= 1) r+=1; 
 if (y <= 1) r-=1;
 return r;
}

inline int GetResult2(uint32 A, uint32 B, uint32 C, uint32 D, uint32 E) 
{
 int x = 0; 
 int y = 0;
 int r = 0;
 if (A == C) x+=1; else if (B == C) y+=1;
 if (A == D) x+=1; else if (B == D) y+=1;
 if (x <= 1) r-=1; 
 if (y <= 1) r+=1;
 return r;
}


inline uint32 RowSharpen (uint32 A, uint32 B, uint32 C)
{
   if (B != C || A != C)
   {
        register GREEN_MASK = 0x07E007E0;
        register int32 x =  (B & GREEN_MASK) >> 5;
        register int32 y = ((A & GREEN_MASK) +
                            (C & GREEN_MASK) >> 7);
        x = x + (x>>1) - (y);
        if (x < 0) x = 0;
        else if (x > 63) x = 63;
        GREEN_MASK = ~GREEN_MASK;
        return ((B & GREEN_MASK) | (x << 5));
   }
   else return B;
}

inline uint32 INTERPOLATE(uint32 A, uint32 B)
{
    if (A !=B)
    {
       register uint32 colorMask = 0xF7DEF7DE;
       register uint32 lowPixelMask = 0x08210821;
       return ( ((A & colorMask) >> 1) + ((B & colorMask) >> 1) + (A & B & lowPixelMask) );
    }
    else return A;
}


inline uint32 Q_INTERPOLATE(uint32 A, uint32 B, uint32 C, uint32 D)
{
        register uint32 qcolorMask = 0xE79CE79C;
        register uint32 qlowpixelMask = 0x18631863;
        register uint32 x = ((A & qcolorMask) >> 2) +
                            ((B & qcolorMask) >> 2) +
                            ((C & qcolorMask) >> 2) +
                            ((D & qcolorMask) >> 2);
        register uint32 y = (A & qlowpixelMask) +
                            (B & qlowpixelMask) +
                            (C & qlowpixelMask) +
                            (D & qlowpixelMask);
        y = (y>>2) & qlowpixelMask;
        return x+y;
}

#ifdef MMX
extern "C" void _2xSaILine  (uint8 *srcPtr, uint32 srcPitch, uint32 width,
                        uint32 dstOffset, uint32 dstPitch, uint16 dstSegment);
int mmx_cpu = 1;
#endif
/*ONLY use with 640x480x16 or higher resolutions*/
/*Only use this if 2*width * 2*height fits on the current screen*/
void InterpolateV2(uint8 *srcPtr, uint32 srcPitch,
	     uint8 *deltaPtr,
	     BITMAP *dstBitmap, int width, int height)
{
    uint32 dstPitch = dstBitmap->w * 2;
    uint32 line;
    uint32 x_offset;
    uint32 dP;
    uint16 *bP;
    uint32 inc_bP;
    line = (screen_height - height * 2) >> 1;

#ifdef MMX
    if (mmx_cpu && width != 512)
    {
        x_offset = (screen_width - width * 2);
	for (height; height; height-=1)
	{

	    bP = (uint16 *) srcPtr;
	    dP = bmp_write_line (dstBitmap, line) + x_offset;
            _2xSaILine  ((uint8 *) bP, srcPitch, width, dP, dstPitch, dstBitmap->seg);
            line += 2;
	    srcPtr += srcPitch;
        }
    }
    else
    {
#endif
        if (width == 512)
        {
                x_offset = (screen_width - width);
                inc_bP = 2;
        }
        else
        {
                x_offset = (screen_width - width * 2);
                inc_bP = 1;
        }

        _farsetsel (dstBitmap->seg);
        uint32 Nextline = srcPitch >> 1;
#ifdef SHARPEN
        uint32 TempColor = 0;
        uint32 Sharpened = 0;
#endif
        for (height; height; height-=1)
	{
	    bP = (uint16 *) srcPtr;
	    dP = bmp_write_line (dstBitmap, line) + x_offset;
            for (uint32 finish = width; finish; finish -= inc_bP )
            {

                register uint32 colorA, colorB;
                uint32 colorC, colorD,
                       colorE, colorF, colorG, colorH,
                       colorI, colorJ, colorK, colorL,
                       colorM, colorN, colorO, colorP;
                uint32 product, product1, product2;


//---------------------------------------
// Map of the pixels:                    I|E F|J
//                                       G|A B|K
//                                       H|C D|L
//                                       M|N O|P
                colorI = *(bP- Nextline - 1);
                colorE = *(bP- Nextline);
                colorF = *(bP- Nextline + 1);
                colorJ = *(bP- Nextline + 2);

                colorG = *(bP - 1);
                colorA = *(bP);
                colorB = *(bP + 1);
                colorK = *(bP + 2);

                colorH = *(bP + Nextline - 1);
                colorC = *(bP + Nextline);
                colorD = *(bP + Nextline + 1);
                colorL = *(bP + Nextline + 2);

                colorM = *(bP + Nextline + Nextline - 1);
                colorN = *(bP + Nextline + Nextline);
                colorO = *(bP + Nextline + Nextline + 1);
                colorP = *(bP + Nextline + Nextline + 2);


                        if ((colorA == colorD) && (colorB != colorC))
                        {
                           if ( ((colorA == colorE) && (colorB == colorL)) ||
                                ((colorA == colorC) && (colorA == colorF) && (colorB != colorE) && (colorB == colorJ)) )
                           {
                              product = colorA;
                           }
                           else
                           {
                              product = INTERPOLATE(colorA, colorB);
                           }

                           if (((colorA == colorG) && (colorC == colorO)) ||
                               ((colorA == colorB) && (colorA == colorH) && (colorG != colorC) && (colorC == colorM)) )
                           {
                              product1 = colorA;
                           }
                           else
                           {
                              product1 = INTERPOLATE(colorA, colorC);
                           }
                           product2 = colorA;
                        }
                        else
                        if ((colorB == colorC) && (colorA != colorD))
                        {
                           if (((colorB == colorF) && (colorA == colorH)) ||
                               ((colorB == colorE) && (colorB == colorD) && (colorA != colorF) && (colorA == colorI)) )
                           {
                              product = colorB;
                           }
                           else
                           {
                              product = INTERPOLATE(colorA, colorB);
                           }

                           if (((colorC == colorH) && (colorA == colorF)) ||
                               ((colorC == colorG) && (colorC == colorD) && (colorA != colorH) && (colorA == colorI)) )
                           {
                              product1 = colorC;
                           }
                           else
                           {
                              product1 = INTERPOLATE(colorA, colorC);
                           }
                           product2 = colorB;
                        }
                        else
                        if ((colorA == colorD) && (colorB == colorC))
                        {
                           if (colorA == colorB)
                           {
                              product = colorA;
                              product1 = colorA;
                              product2 = colorA;
                           }
                           else
                           {
                              register int r = 0;
                              product1 = INTERPOLATE(colorA, colorC);
                              product = INTERPOLATE(colorA, colorB);

                              r += GetResult1 (colorA, colorB, colorG, colorE, colorI);
                              r += GetResult2 (colorB, colorA, colorK, colorF, colorJ);
                              r += GetResult2 (colorB, colorA, colorH, colorN, colorM);
                              r += GetResult1 (colorA, colorB, colorL, colorO, colorP);

                              if (r > 0)
                                  product2 = colorA;
                              else
                              if (r < 0)
                                  product2 = colorB;
                              else
                              {
                                  product2 = Q_INTERPOLATE(colorA, colorB, colorC, colorD);
                              }
                           }
                        }
                        else
                        {
                           product2 = Q_INTERPOLATE(colorA, colorB, colorC, colorD);

                           if ((colorA == colorC) && (colorA == colorF) && (colorB != colorE) && (colorB == colorJ))
                           {
                              product = colorA;
                           }
                           else
                           if ((colorB == colorE) && (colorB == colorD) && (colorA != colorF) && (colorA == colorI))
                           {
                              product = colorB;
                           }
                           else
                           {
                              product = INTERPOLATE(colorA, colorB);
                           }

                           if ((colorA == colorB) && (colorA == colorH) && (colorG != colorC) && (colorC == colorM))
                           {
                              product1 = colorA;
                           }
                           else
                           if ((colorC == colorG) && (colorC == colorD) && (colorA != colorH) && (colorA == colorI))
                           {
                              product1 = colorC;
                           }
                           else
                           {
                              product1 = INTERPOLATE(colorA, colorC);
                           }
                        }
#ifndef SHARPEN
                        product = colorA | (product << 16);
                        product1 = product1 | (product2 << 16);
			_farnspokel (dP, product);
			_farnspokel (dP + dstPitch, product1);
#else
                        Sharpened = RowSharpen ( TempColor, colorA, product);
                        Sharpened |= (RowSharpen ( colorA, product, colorB) << 16);

			_farnspokel (dP, Sharpened);
			_farnspokel (dP + dstPitch, product1| (product2 << 16));

                        TempColor = product;
#endif
                    bP += inc_bP;
                    dP += 4;
                }//end of for ( finish= width etc..)
#ifdef SHARPEN
            TempColor = 0;
#endif

            line += 2;
            srcPtr += srcPitch;
            deltaPtr += srcPitch << 1;
	}; //endof: for (height; height; height--)
#ifdef MMX
    }
#endif
}





