/********************************************************/
/* Virtual Applications' OpenGL Renderer Source Code    */
/********************************************************/
/* You're allowed to make changes to this file. If the  */
/* changes are constructive, you can e-mail me with the */
/* patch to be applied to this source. You'll be given  */
/* full credits for changes you've made.                */
/********************************************************/
/* Contributors:                                        */
/* Jean-Philip Desjardins (First Code and Maintainer)   */
/* Yves Rioux (Some Sprite Fixes and Fl/Cl Code)		*/
/********************************************************/

#include "COMMON.H"

extern HANDLE VOGL_hModule;

static VOGL_CONFIG Default = 
{
	0,
	BST_UNCHECKED,
	0,
	BST_CHECKED,
};

VOGL_CONFIG VOGL_Config;

char *lpstrResolution[] = 
{
	"320x240",
	"512x384",
	"640x480",
	"800x600",
	"1024x768",
	"1152x864",
	"1280x768",
	"1024x960",
	"1280x1024",
	NULL
};

char *lpstrDistances[] =
{
	"Full",
	"Far",
	"Close",
	NULL
};

int VOGL_nReturn = 0;

void CONFIG_LoadConfiguration()
{
	DWORD dwDisp;
	DWORD dwSize = sizeof(VOGL_CONFIG);
	HKEY hkey;
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Virtual Applications\\xBUILD OpenGL Renderer", NULL, NULL, NULL, KEY_ALL_ACCESS, NULL, &hkey, &dwDisp);
	if(dwDisp & REG_CREATED_NEW_KEY)
	{
		memmove(&VOGL_Config, &Default, sizeof(VOGL_CONFIG));
		RegCloseKey(hkey);
		RegDeleteKey(HKEY_LOCAL_MACHINE, "Software\\Virtual Applications\\xBUILD OpenGL Renderer");
		return;
	}
	RegQueryValueEx(hkey, "Config", NULL, NULL, (unsigned char*)&VOGL_Config, &dwSize);
	RegCloseKey(hkey);
}

void CONFIG_SaveConfiguration()
{
	HKEY hkey;
	DWORD dwSize = sizeof(VOGL_CONFIG);
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Virtual Applications\\xBUILD OpenGL Renderer", NULL, NULL, NULL, KEY_ALL_ACCESS, NULL, &hkey, NULL);
	RegSetValueEx(hkey, "Config", NULL, NULL, (unsigned char*)&VOGL_Config, dwSize);
	RegCloseKey(hkey);
}

int WINAPI CfgProc(HWND hWnd, unsigned int uiMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uiMsg)
	{
	case WM_COMMAND:
		switch(wParam)
		{
		case 2:
			DestroyWindow(hWnd);
			break;
		case IDOK:
			VOGL_nReturn = 1;
			DestroyWindow(hWnd);
			break;
		}
		break;
	case WM_DESTROY:
		VOGL_Config.nResolution = SendDlgItemMessage(hWnd, IDC_RESOLUTION, CB_GETCURSEL, 0, 0);
		VOGL_Config.nFullscreen = SendDlgItemMessage(hWnd, IDC_FS, BM_GETCHECK, 0, 0);
		VOGL_Config.nDistance = SendDlgItemMessage(hWnd, IDC_DISTANCES, CB_GETCURSEL, 0, 0);
		VOGL_Config.nMipmapping = SendDlgItemMessage(hWnd, IDC_MIPMAP, BM_GETCHECK, 0, 0);
		break;
	case WM_INITDIALOG:
		int i = 0;
		SendDlgItemMessage(hWnd, IDC_RESOLUTION, CB_RESETCONTENT, 0, 0);
		while(1)
		{
			if(lpstrResolution[i] == NULL)
			{
				break;
			}
			SendDlgItemMessage(hWnd, IDC_RESOLUTION, CB_ADDSTRING, 0, (LPARAM)lpstrResolution[i]);
			i++;
		}
		i = 0;
		while(1)
		{
			if(lpstrDistances[i] == NULL)
			{
				break;
			}
			SendDlgItemMessage(hWnd, IDC_DISTANCES, CB_ADDSTRING, 0, (LPARAM)lpstrDistances[i]);
			i++;
		}
		SendDlgItemMessage(hWnd, IDC_RESOLUTION, CB_SETCURSEL, VOGL_Config.nResolution, 0);
		SendDlgItemMessage(hWnd, IDC_DISTANCES, CB_SETCURSEL, VOGL_Config.nDistance, 0);
		SendDlgItemMessage(hWnd, IDC_FS, BM_SETCHECK, VOGL_Config.nFullscreen, 0);
		SendDlgItemMessage(hWnd, IDC_MIPMAP, BM_SETCHECK, VOGL_Config.nMipmapping, 0);
		break;
	}
	return FALSE;
}

void CONFIG_Start()
{
	CONFIG_LoadConfiguration();
	DialogBox((HMODULE)VOGL_hModule, MAKEINTRESOURCE(IDD_CONFIG), NULL, CfgProc);
	if(VOGL_nReturn)
	{
		CONFIG_SaveConfiguration();
	}
}