#include "COMMON.H"

struct MD2HEADER
{
	int nMagic;
	int nVersion;
	int nSkinWidth;
	int nSkinHeight;
	int nFrameSize;
	int nNumSkins;
	int nNumVertices;
	int nNumTexCoords;
	int nNumTriangles;
	int nNumglCommands;
	int nNumFrames;
	int nOffsetSkins;
	int nOffsetTexCoords;
	int nOffsetTriangles;
	int nOffsetFrame;
	int nOffsetglCommands;
	int nOffsetEnd;
};

struct MD2VERTEX
{
	unsigned _int8	bVertex[3];
	unsigned _int8	bLightNormalIdx;
};

struct MD2TEXCOORDS
{
	short s;
	short t;
};

struct MD2TRIANGLE
{
	short	wVertexIndice[3];
	short	wTextureIndice[3];
};

struct MD2FRAME
{
	float				fScale[3];
	float				fTranslate[3];
	char				strName[16];
	MD2VERTEX			Vertice[2048];
};

struct MD2GLCOMMAND
{
	float s, t;
	int nVertexIndex;
};

struct PCXHEADER
{
	unsigned _int8		bManufacturer;
	unsigned _int8		bVersion;
	unsigned _int8		bEncoding;
	unsigned _int8		bBPP;
	unsigned _int16		wXMin;
	unsigned _int16		wYMin;
	unsigned _int16		wXMax;
	unsigned _int16		wYMax;
	unsigned _int16		wHDPI;
	unsigned _int16		wVDPI;
	unsigned _int8		Palette[48];
	unsigned _int8		bReversed;
	unsigned _int8		bPlanes;
	unsigned _int16		wBPL;
	unsigned _int16		wPaletteInfo;
	unsigned _int16		wHSize;
	unsigned _int16		wVSize;
	unsigned _int8		Filler[54];
};

//This is taking lots of memory, dynamic someone?
MD2HEADER		md2_header;
char			md2_skin[64][32];
MD2TEXCOORDS	md2_texcoord[2048];
MD2TRIANGLE		md2_triangle[4096];
MD2FRAME		md2_frame[512];
MD2GLCOMMAND	md2_glcommand[2048];
unsigned int	md2_skintex[32];

void MD2_LoadSkin(char *strFile, int nNumber)
{
	FILE *stream;
	PCXHEADER h;
	int nCX, nCY, ptr, i, rc;
	unsigned _int8 *data;
	unsigned _int8 *palette;
	unsigned _int8 *picture;
	unsigned _int8 pind;
	stream = fopen(strFile, "rb");
	if(stream == NULL)
	{
		return;
	}
	fread(&h, sizeof(PCXHEADER), 1, stream);
	if(h.bBPP != 8 || h.bPlanes != 1)
	{
		//Doesn't support less than 8 bits per pixel or more than 1 bit plane
		fclose(stream);
		return;
	}
	nCX = h.wXMax - h.wXMin + 1;
	nCY = h.wYMax - h.wYMin + 1;
	data = new unsigned _int8[nCX * nCY];
	for(i = 0; i < nCX * nCY;)
	{
		fread(&pind, 1, 1, stream);
		if((pind & 0xC0) == 0xC0)
		{
			rc = pind & 0x3F;
			fread(&pind, 1, 1, stream);
			memset(data + i, pind, rc);
			i += rc;
		}
		else
		{
			data[i] = pind;
			i++;
		}
	}
	fread(&pind, 1, 1, stream);
	if(pind != 0x0C)
	{
		//We don't have a palette...
		delete data;
		fclose(stream);
		return;
	}
	palette = new unsigned _int8[0x300];
	fread(palette, 0x300, 1, stream);
	picture = new unsigned _int8[nCX * nCY * 3];
	ptr = 0;
	for(i = 0; i < (nCX * nCY); i++)
	{
		picture[ptr++] = palette[(data[i] * 3) + 0];
		picture[ptr++] = palette[(data[i] * 3) + 1];
		picture[ptr++] = palette[(data[i] * 3) + 2];
	}
	glGenTextures(1, &md2_skintex[0]);
	glBindTexture(GL_TEXTURE_2D, md2_skintex[0]);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, nCX, nCY, GL_RGB, GL_UNSIGNED_BYTE, picture);
	delete data;
	delete palette;
	delete picture;
	fclose(stream);
}

void MD2_Load(char *strFile)
{
	int i;
	FILE *stream;
	stream = fopen(strFile, "rb");
	fread(&md2_header, sizeof(MD2HEADER), 1, stream);
	fseek(stream, md2_header.nOffsetSkins, SEEK_SET);
	for(i = 0; i < md2_header.nNumSkins; i++)
	{
		fread(md2_skin[i], 64, 1, stream);
		MD2_LoadSkin(md2_skin[i], i);
	}
	fseek(stream, md2_header.nOffsetTexCoords, SEEK_SET);
	for(i = 0; i < md2_header.nNumTexCoords; i++)
	{
		fread(&md2_texcoord[i], sizeof(MD2TEXCOORDS), 1, stream);
	}
	fseek(stream, md2_header.nOffsetTriangles, SEEK_SET);
	for(i = 0; i < md2_header.nNumTriangles; i++)
	{
		fread(&md2_triangle[i], sizeof(MD2TRIANGLE), 1, stream);
	}
	fseek(stream, md2_header.nOffsetFrame, SEEK_SET);
	for(i = 0; i < md2_header.nNumFrames; i++)
	{
		fread(&md2_frame[i], sizeof(MD2FRAME) - (sizeof(MD2VERTEX) * 2048), 1, stream);
		fread(&md2_frame[i].Vertice, sizeof(MD2VERTEX) * md2_header.nNumVertices, 1, stream);
	}
	fseek(stream, md2_header.nOffsetglCommands, SEEK_SET);
	for(i = 0; i < md2_header.nNumglCommands; i++)
	{
		fread(&md2_glcommand[i], sizeof(MD2GLCOMMAND), 1, stream);
	}
	fclose(stream);
}

void MD2_CreateDisplayList(int *DL, float fScaleFactor)
{
	int nFrame = 0;
	int i;
	*DL = glGenLists(1);
	glNewList(*DL, GL_COMPILE);
	glPushMatrix();
	glTranslatef(md2_frame[nFrame].fTranslate[0], md2_frame[nFrame].fTranslate[1], md2_frame[nFrame].fTranslate[2]);
	glScalef(md2_frame[nFrame].fScale[0] * fScaleFactor, md2_frame[nFrame].fScale[1] * fScaleFactor, md2_frame[nFrame].fScale[2] * fScaleFactor); 
	glBindTexture(GL_TEXTURE_2D, md2_skintex[0]);
	glBegin(GL_TRIANGLES);
	{
		for(i = 0; i < md2_header.nNumTriangles; i++)
		{
			MD2VERTEX *v0, *v1, *v2;
			MD2TEXCOORDS *t0, *t1, *t2;
			v0 = &md2_frame[nFrame].Vertice[md2_triangle[i].wVertexIndice[0]];
			v1 = &md2_frame[nFrame].Vertice[md2_triangle[i].wVertexIndice[1]];
			v2 = &md2_frame[nFrame].Vertice[md2_triangle[i].wVertexIndice[2]];
			t0 = &md2_texcoord[md2_triangle[i].wTextureIndice[0]];
			t1 = &md2_texcoord[md2_triangle[i].wTextureIndice[1]];
			t2 = &md2_texcoord[md2_triangle[i].wTextureIndice[2]];

			glTexCoord2f((float)t0->s / (float)md2_header.nSkinWidth, (float)t0->t / (float)md2_header.nSkinHeight);
			glVertex3i(v0->bVertex[0], v0->bVertex[1], v0->bVertex[2]);

			glTexCoord2f((float)t1->s / (float)md2_header.nSkinWidth, (float)t1->t / (float)md2_header.nSkinHeight);
			glVertex3i(v1->bVertex[0], v1->bVertex[1], v1->bVertex[2]);

			glTexCoord2f((float)t2->s / (float)md2_header.nSkinWidth, (float)t2->t / (float)md2_header.nSkinHeight);
			glVertex3i(v2->bVertex[0], v2->bVertex[1], v2->bVertex[2]);
		}
	}
	glEnd();
	glPopMatrix();
	glEndList();
}

void MD2_CreateDisplayListForMesh(char *strFile, int *DL, float fScaleFactor)
{
	MD2_Load(strFile);	
	MD2_CreateDisplayList(DL, fScaleFactor);
}
