/********************************************************/
/* Virtual Applications' OpenGL Renderer Source Code    */
/********************************************************/
/* You're allowed to make changes to this file. If the  */
/* changes are constructive, you can e-mail me with the */
/* patch to be applied to this source. You'll be given  */
/* full credits for changes you've made.                */
/********************************************************/
/* Contributors:                                        */
/* Jean-Philip Desjardins (First Code and Maintainer)   */
/* Yves Rioux (Some Sprite Fixes and Fl/Cl Code)		*/
/********************************************************/

#include "COMMON.H"

#define VERSION "v0.02"

#define DLLEXPORT __declspec(dllexport)

HANDLE							VOGL_hModule;
HDC								VOGL_hDC;
HGLRC							VOGL_hRC;
HWND							VOGL_hWnd;
extern XBUILD_ART_TILEDATA*		VOGL_Tiles[MAXTILES];
extern int						VOGL_TilesCreated[MAXTILES];
int								VOGL_nMesh[MAXTILES];
extern VOGL_CONFIG				VOGL_Config;
int								VOGL_nWindowWidth;
int								VOGL_nWindowHeight;
extern XBUILD_MAP				VOGL_Map;
extern unsigned int				VOGL_nTexture[MAXTILES];
extern LISTSPRITE*				VOGL_pSpriteBuffer;

int VOGL_Height[] =
{
	240,
	384,
	480,
	600,
	768,
	864,
	768,
	960,
	1024,
};

int VOGL_Width[] =
{
	320,
	512,
	640,
	800,
	1024,
	1152,
	1280,
	1024,
	1280,

};

static PIXELFORMATDESCRIPTOR pfd = 
{
	sizeof(PIXELFORMATDESCRIPTOR),
	1,
	PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
	PFD_TYPE_RGBA,
	32,
	0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0,
	0,
	32,
	0,
	0,
	PFD_MAIN_PLANE,
	0,
	0, 0, 0
};

void center_window(HWND hWnd)
{
	int resx = GetDeviceCaps(GetDC(hWnd), HORZRES);
	int resy = GetDeviceCaps(GetDC(hWnd), VERTRES);
	int wndx;
	int wndy;
	RECT wnd;
	GetWindowRect(hWnd, &wnd);
	wndx = wnd.right - wnd.left;
	wndy = wnd.bottom - wnd.top;
	SetWindowPos(hWnd, NULL, (resx - wndx) / 2, (resy - wndy) / 2, NULL, NULL, SWP_NOZORDER | SWP_NOSIZE);
}

extern "C" void Resize(int nWidth, int nHeight)
{
	glViewport(0, 0, nWidth, nHeight);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, (GLfloat)nWidth/(GLfloat)nHeight, 3.0f, 2000.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

extern "C" int InitializeOpenGL()
{
	RECT rc;
	GetClientRect(VOGL_hWnd, &rc);
	Resize(rc.right, rc.bottom);
	glEnable(GL_TEXTURE_2D);
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	return 1;
}

extern "C" int Initialize(HWND hWnd)
{
	DEVMODE dm;
	unsigned int pf;
	CONFIG_LoadConfiguration();
	VOGL_nWindowWidth = VOGL_Width[VOGL_Config.nResolution];
	VOGL_nWindowHeight = VOGL_Height[VOGL_Config.nResolution];
	VOGL_hWnd = hWnd;
	SetWindowPos(VOGL_hWnd, NULL, 0, 0, VOGL_nWindowWidth, VOGL_nWindowHeight, NULL);
	VOGL_hDC = GetDC(VOGL_hWnd);
	dm.dmSize = sizeof(DEVMODE);
	dm.dmPelsWidth = VOGL_nWindowWidth;
	dm.dmPelsHeight = VOGL_nWindowHeight;
	dm.dmBitsPerPel = 16;
	dm.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
	pf = ChoosePixelFormat(VOGL_hDC, &pfd);
	SetPixelFormat(VOGL_hDC, pf, &pfd);
	VOGL_hRC = wglCreateContext(VOGL_hDC);
	wglMakeCurrent(VOGL_hDC, VOGL_hRC);
	if(VOGL_Config.nFullscreen)
	{
		ChangeDisplaySettings(&dm, CDS_FULLSCREEN);
	}
	else
	{
		center_window(hWnd);
	}
	InitializeOpenGL();
	memset(VOGL_TilesCreated, 0, sizeof(int) * MAXTILES);
	memset(VOGL_nTexture, 0, sizeof(int) * MAXTILES);
	memset(VOGL_nMesh, 0xFF, sizeof(int) * MAXTILES);
	return 1;
}

extern "C" void Destroy()
{
	int i;
	for(i = 0; i < MAXTILES; i++)
	{
		if(VOGL_Tiles[i] != NULL)
		{
			delete VOGL_Tiles[i];
		}
		if(VOGL_nTexture[i] != NULL)
		{
			glDeleteTextures(1, &VOGL_nTexture[i]);
		}
	}
	if(VOGL_pSpriteBuffer != NULL)
	{
		delete VOGL_pSpriteBuffer;
	}
	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(VOGL_hRC);
	ReleaseDC(VOGL_hWnd, VOGL_hDC);
	ChangeDisplaySettings(NULL, 0);
}

extern "C" void Config()
{
	CONFIG_Start();
}

extern "C" void About()
{
	char str[256];
	sprintf(str, "Virtual Applications OpenGL Renderer %s\n\nBy Jean-Philip Desjardins\nand\nYves Rioux\n\nFor the xBUILD engine.", VERSION);
	MessageBox(NULL, str, "About", MB_ICONINFORMATION);
}

extern "C" void *GetMapAddress()
{
	return (void*)&VOGL_Map;
}

extern "C" int IsFullScreen()
{
	CONFIG_LoadConfiguration();
	return VOGL_Config.nFullscreen;
}

extern "C" int GetWidth()
{
	CONFIG_LoadConfiguration();
	return VOGL_Width[VOGL_Config.nResolution];
}

extern "C" int GetHeight()
{
	CONFIG_LoadConfiguration();
	return VOGL_Height[VOGL_Config.nResolution];
}

extern "C" void SetMeshForTile(int nTile, char *lpstrFilename, float fScaleFactor)
{
	char strExt[16];
	_splitpath(lpstrFilename, NULL, NULL, NULL, strExt);
	if(!stricmp(strExt, ".MD3"))
	{
		MD3_CreateDisplayListForMesh(lpstrFilename, &VOGL_nMesh[nTile], fScaleFactor); 
	}
	else if(!stricmp(strExt, ".MD2"))
	{
		MD2_CreateDisplayListForMesh(lpstrFilename, &VOGL_nMesh[nTile], fScaleFactor);
	}
}

extern "C" void TakeScreenShot(char *lpstrFilename)
{
	RECT rc;
	int i;
	unsigned _int8 temp;
	unsigned _int8 *data;
	GetClientRect(VOGL_hWnd, &rc);
	data = new unsigned _int8[rc.right * rc.bottom * 4];
	glReadPixels(0, 0, rc.right, rc.bottom, GL_RGBA, GL_UNSIGNED_BYTE, data);
	for(i = 0; i < (rc.right * rc.bottom * 4); i += 4)
	{
		temp = data[i + 0];
		data[i + 0] = data[i + 2];
		data[i + 2] = temp;
	}
	XBUILD_GFX_SaveImageAsTGA(lpstrFilename, rc.right, rc.bottom, 32, data);
	delete data;
}

XBUILDRENCONTEXT RendOGLContext =
{
	Initialize,
	Config,
	About,
	Destroy,
	DrawMap,
	PrepareMap,
	SwapDisplayBuffers,
	ProcessDisplayList,
	GetMapAddress,
	IsFullScreen,
	GetWidth,
	GetHeight,
	SetMeshForTile,
	TakeScreenShot,
	"OpenGL Renderer",
	"Virtual Applications",
	VERSION
};

//XBUILD RENDERER INITIAL FUNCTIONS

extern "C" void DLLEXPORT GetRendererContext(void* context)
{
	memmove(context, &RendOGLContext, sizeof(XBUILDRENCONTEXT));
}

BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{
	VOGL_hModule = hModule;
	return TRUE;
}
