#include "COMMON.H"

struct TGAHEADER
{
	unsigned char	NumIdChar;
	unsigned char	PaletteType;
	unsigned char	ImageType;
	unsigned short	PaletteOrigin;
	unsigned char	PaletteBitsPerEntry;
	unsigned short	Xpos;
	unsigned short	Ypos;
	unsigned short	Width;
	unsigned short	Height;
	unsigned char	BitsPerPixel;
	unsigned char	Descriptor;
};

XBUILDRENCONTEXT	GFX_Rend;
HWND				GFX_hWnd;
HANDLE				GFX_hThread = NULL;
DWORD				GFX_dwTID = NULL;
int					GFX_nKillThread = NULL;
int					GFX_nX;
int					GFX_nY;
int					GFX_nZ;
int					GFX_nXRot;
int					GFX_nYRot;
XBUILD_MAP			GFX_Map;
int					GFX_nLoadMapDone = 1;
DWORD				GFX_dwFrameTime = 17;
unsigned _int8		GFX_nStatus;
XBUILDDISPLAYLIST	GFX_StatusDisplayList;
XBUILDDISPLAYLIST	GFX_ConsoleDisplayList;
int					GFX_nActive = 0;
void				(*GFX_SystemCommandCallback)(int, void*) = NULL;
void*				GFX_SystemCommandCallbackParam = NULL;

#define				GFX_MESSAGE_PREPARE			0xBEF0
#define				GFX_MESSAGE_SETMODEL		0xBEF1
#define				GFX_MESSAGE_SCREEN			0xBEF2

#define				XBUILD_GFX_LAYER_3DMAP		0x01
#define				XBUILD_GFX_LAYER_CONSOLE	0x02
#define				XBUILD_GFX_LAYER_STATUS		0x04

void (*GetRendererContextDLL)(XBUILDRENCONTEXT*);

struct GFX_MESHSEND
{
	char*	lpstrFilename;
	float	fScaleFactor;
};

int WINAPI GFX_WndProc(HWND hWnd, unsigned int uiMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uiMsg)
	{
	case GFX_MESSAGE_PREPARE:
		GFX_Rend.PrepareMap(&GFX_Map);
		GFX_nLoadMapDone = 1;
		break;
	case GFX_MESSAGE_SETMODEL:
		GFX_MESHSEND *s;
		s = (GFX_MESHSEND*)lParam;
		GFX_Rend.SetMeshForTile(wParam, s->lpstrFilename, s->fScaleFactor);
		break;
	case GFX_MESSAGE_SCREEN:
		GFX_Rend.TakeScreenShot((char*)lParam);
		break;
	case WM_ACTIVATE:
		//GFX_nActive = wParam;
		GFX_nActive = 1;
		break;
	case WM_SYSCOMMAND:
		if(GFX_SystemCommandCallback != NULL)
		{
			GFX_SystemCommandCallback(wParam, GFX_SystemCommandCallbackParam);
		}
		break;
	case WM_DESTROY:
		GFX_Rend.Destroy();
		break;
	case WM_INITDIALOG:
		GFX_Rend.Initialize(hWnd);
		break;
	case MM_MCINOTIFY:
		MUSIC_MCICallback(wParam, lParam);
		break;
	}
	return FALSE;
}

unsigned long WINAPI GFX_WindowThread(void *)
{
	MSG msg;
	DWORD dwTime;
	DWORD dwCurrent;
	if(GFX_Rend.IsFullScreen())
	{
		GFX_hWnd = CreateDialog((HMODULE)XBUILD_hMod, MAKEINTRESOURCE(IDD_OUTPUTFS), NULL, GFX_WndProc);
	}
	else
	{
		GFX_hWnd = CreateDialog((HMODULE)XBUILD_hMod, MAKEINTRESOURCE(IDD_OUTPUT), NULL, GFX_WndProc);
	}
	ShowCursor(FALSE);
	while(!GFX_nKillThread)
	{
		dwTime = GetTickCount();
		PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE);
		if(!IsDialogMessage(GFX_hWnd, &msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		if(GFX_nActive)
		{
			GFX_Rend.SwapDisplayBuffers();
			if(GFX_nStatus & XBUILD_GFX_LAYER_3DMAP)
			{
				GFX_Rend.DrawMap(GFX_nX, GFX_nY, GFX_nZ, GFX_nXRot, GFX_nYRot);
			}
			if(GFX_nStatus & XBUILD_GFX_LAYER_STATUS)
			{
				GFX_Rend.ProcessDisplayList((void*)&GFX_StatusDisplayList);
			}
			if(GFX_nStatus & XBUILD_GFX_LAYER_CONSOLE)
			{
				GFX_Rend.ProcessDisplayList((void*)&GFX_ConsoleDisplayList);
			}
		}
		dwCurrent = GetTickCount();
		if(dwCurrent > dwTime + GFX_dwFrameTime)
		{
			GFX_dwFrameTime++;
		}
		else
		{
			Sleep((dwTime + GFX_dwFrameTime) - dwCurrent);
			if(GFX_dwFrameTime != 17)
			{
				GFX_dwFrameTime--;
			}
		}
	}
	return 0xDEADBEEF;
}

extern "C" HWND DLLEXPORT XBUILD_GFX_Initialize(char *lpstrRendererFile)
{
	//Load our renderer
	HMODULE hMod;
	int nWait = 0;
	hMod = LoadLibrary(lpstrRendererFile);
	if(hMod == NULL)
	{
		return 0;
	}
	GetRendererContextDLL = (void (*)(XBUILDRENCONTEXT*))GetProcAddress(hMod, "GetRendererContext");
	if(GetRendererContextDLL == NULL)
	{
		return 0;
	}
	GetRendererContextDLL(&GFX_Rend);
	//Spawn a thread
	GFX_hThread = CreateThread(NULL, NULL, GFX_WindowThread, NULL, NULL, &GFX_dwTID);
	while(!GFX_hWnd)
	{
		Sleep(100);
		nWait++;
		if(nWait > 100)
		{
			MessageBox(NULL, "A problem occured when trying to initialize the renderer.", "xBUILD", 16);
			break;
		}
	}
	return GFX_hWnd;
}

extern "C" int DLLEXPORT XBUILD_GFX_Update(int x, int y, int z, int xrot, int yrot)
{
	GFX_nX = x;
	GFX_nY = y;
	GFX_nZ = z;
	GFX_nXRot = xrot;
	GFX_nYRot = yrot;
	return GFX_dwFrameTime;
}

extern "C" void DLLEXPORT XBUILD_GFX_Destroy()
{
	if(GFX_hWnd)
	{
		DestroyWindow(GFX_hWnd);
	}
	GFX_nKillThread = 1;
	WaitForSingleObject(GFX_hThread, INFINITE);
	GFX_Rend.Destroy();
}

extern "C" void DLLEXPORT XBUILD_GFX_SetMap(XBUILD_MAP *lpMap)
{
	GFX_nLoadMapDone = 0;
	memmove(&GFX_Map, lpMap, sizeof(XBUILD_MAP));
	SendMessage(GFX_hWnd, GFX_MESSAGE_PREPARE, NULL, NULL);
	while(!GFX_nLoadMapDone)
	{

	}
}

extern "C" void DLLEXPORT XBUILD_GFX_SetDisplayList(int nLayer, XBUILDDISPLAYLIST *d)
{
	switch(nLayer)
	{
	case XBUILD_GFX_LAYER_STATUS:
		memcpy(&GFX_StatusDisplayList, d, sizeof(XBUILDDISPLAYLIST));
		break;
	case XBUILD_GFX_LAYER_CONSOLE:
		memcpy(&GFX_ConsoleDisplayList, d, sizeof(XBUILDDISPLAYLIST));
		break;
	}
}

extern "C" void DLLEXPORT XBUILD_GFX_SetStatus(int nStatus)
{
	//Status Bits
	//11111111111
	//      |||||--- Map Visible
	//      ||||---- Console Visible 
	//      |||----- Status Visible
	//      ||------ Menu Visible
	//      |------- Sprites Visible
	GFX_nStatus = nStatus;
}

extern "C" int DLLEXPORT XBUILD_GFX_GetScreenWidth()
{
	return GFX_Rend.GetWidth();
}

extern "C" int DLLEXPORT XBUILD_GFX_GetScreenHeight()
{
	return GFX_Rend.GetHeight();
}

extern "C" void DLLEXPORT *XBUILD_GFX_GetMapAddress()
{
	return GFX_Rend.GetMapAddress();
}

extern "C" int DLLEXPORT XBUILD_GFX_AddTileToDisplayList(int nTile, int nX, int nY, XBUILDDISPLAYLIST *d, int nPos)
{
	XBUILDDISPLAYELEMENT_TILE DTile;
	DTile.dwTile	= nTile;
	DTile.dwX		= nX;
	DTile.dwY		= nY;
	if(nPos == -1)
	{
		nPos = d->nElementCount++;
	}
	memcpy(d->Element[nPos].pData, &DTile, sizeof(XBUILDDISPLAYELEMENT_TILE));
	d->Element[nPos].dwType = XBUILDDISPLAYLIST_TYPE_TILE;
	return nPos;
}

extern "C" void DLLEXPORT XBUILD_GFX_SetMeshForTile(int nTile, char *lpstrFilename, float fScaleFactor)
{
	GFX_MESHSEND s;
	s.lpstrFilename = lpstrFilename;
	s.fScaleFactor = fScaleFactor;
	SendMessage(GFX_hWnd, GFX_MESSAGE_SETMODEL, nTile, (LPARAM)&s);
}

extern "C" void DLLEXPORT XBUILD_GFX_SetSystemCommandCallback(void *function, void* pParam)
{
	GFX_SystemCommandCallback = (void (*)(int, void*))function;
	GFX_SystemCommandCallbackParam = pParam;
}

extern "C" void DLLEXPORT XBUILD_GFX_TakeScreenShot(char *lpstrFilename)
{
	SendMessage(GFX_hWnd, GFX_MESSAGE_SCREEN, NULL, (LPARAM)lpstrFilename);
}

extern "C" void DLLEXPORT XBUILD_GFX_SaveImageAsTGA(char *lpstrFilename, int nCX, int nCY, int nBPP, unsigned _int8 *data)
{
	TGAHEADER h;
	FILE *stream;
	int nSize;
	memset(&h, 0, sizeof(TGAHEADER));
	h.ImageType = 2;
	h.Width = nCX;
	h.Height = nCY;
	h.BitsPerPixel = nBPP;
	stream = fopen(lpstrFilename, "wb");
	fwrite(&h, sizeof(TGAHEADER), 1, stream);
	nSize = nCX * nCY * (nBPP / 8);
	fwrite(data, nSize, 1, stream);
	fclose(stream);
}